'''
Simple script to convert the MNIST data downloaded from

http://yann.lecun.com/exdb/mnist/

to a format acceptable to the sample svmma (SVM training using SPG)
code.

Author: Paulo J. S. Silva
'''

import sys
from subprocess import call
import scipy as sp

from readMNIST import readMNIST

def read_file(images_name, labels_name, total):
    print 'Reading', images_name, '...',
    try:
        imgs, lbls = readMNIST(images_name, labels_name, total, 0)
    except:
        print >>sys.stderr, 'Bad file format'
        sys.exit(1)
    m, n, nimages = imgs.shape
    print 'with', nimages, 'images.'

    print 'Flattenning images...',
    images = []
    for i in xrange(nimages):
        image = imgs[:,:,i]
        image = image.flatten()
        image = sp.hstack( (lbls[i], image) )
        images.append(image)
    images = sp.array(images)
    print 'Done.'

    # Definig classes
    print 'Defining classes...',
    eights = (images[:, 0] == 8)
    not_eights = sp.logical_not(eights)
    images[eights,0] = -1
    images[not_eights, 0] = 1
    print 'Done.'

    return images

if __name__ == '__main__':
    # Fix a random seed to get the same resutls
    sp.random.seed(1)

    # Read training data
    train_images = '../../svm_data/mnist/train-images-idx3-ubyte'
    train_labels = '../../svm_data/mnist/train-labels-idx1-ubyte'
    print 'Reading training data...'
    train = read_file(train_images, train_labels, 60000)
    sp.random.shuffle(train)

    # Read test data
    print 'Reading test data...'
    test_images = '../../svm_data/mnist/t10k-images-idx3-ubyte'
    test_labels = '../../svm_data/mnist/t10k-labels-idx1-ubyte'
    test = read_file(test_images, test_labels, 10000)

    # Save train and test data
    print 'Saving SVM formatted data...'
    sp.savez('mnist.svm', train=train, test=test)

    # Print statistics
    print 'Done.'
    print 'Train positive:', (train[:,0] == 1).sum()
    print 'Train negative:', (train[:,0] == -1).sum()
    print 'Train total:   ', train.shape[0]
    print 'Test postive:  ',  (test[:,0] == 1).sum()
    print 'Test negative: ',  (test[:,0] == -1).sum()
    print 'Test total:    ',  test.shape[0]


    
