'''
Read the SVM computed by svmma and test it against a test set.
'''

import scipy as sp
from convert_to_svm import sigmoid
import sys


def get_input():
    '''Get command line parameters defining the traind and test data
    and the parameters defining the SVN.
    '''
    try:
        return sys.argv[1], sys.argv[2], eval(sys.argv[3]), eval(sys.argv[4])
    except:
        print 'Usage: test_svm.py train_file test_file sigma C'
        sys.exit(1)

def read_solution(train, sigma, C):
    '''Read the solution given by svmma and creates a SVM classifier.
    '''
    print 'Building the SVM classifier...'
    # Read solution
    index, multiplier = 0, []
    for line in file('solution.txt'):
        multiplier.append(float(line.split()[2]))
        index += 1
    multiplier = sp.array(multiplier)

    # Define the linear part of the SVM
    nonzero = (multiplier != 0.0)
    nzmul = multiplier[nonzero].copy()
    nztrain = train[nonzero].copy()
    def svm_minus_b(x):
        terms = nzmul*nztrain[:,0]*sigmoid(x, nztrain[:,1:], sigma)
        return terms.sum()

    # Estimate b^*
    eps = sp.finfo(sp.float64).eps
    low = eps*multiplier[nonzero].mean()
    up = (1.0 - eps)*C
    inactive_mul = sp.logical_and(low < multiplier, multiplier < up)
    bs = [(1.0 - t[0]*svm_minus_b(t[1:]))/t[0] for t in train[inactive_mul]]
    bs = sp.array(bs)
    b = bs.sum()/len(bs)

    # Define the classification function and return it.
    def svm(x):
        return svm_minus_b(x) + b

    return svm

def test_svm(svm, test):
    '''Test and SVM classifier against a test set.
    '''
    print 'Testing...'
    errors_pos, errors_neg = 0, 0
    pos, neg = 0, 0 
    for sample in test:
        if sample[0] > 0.0:
            pos += 1
            if svm(sample[1:]) <= 0.0:
                errors_pos += 1
        elif sample[0] < 0.0:
            neg += 1
            if svm(sample[1:]) >= 0.0:
                errors_neg += 1
    print 'Errors for positive class', errors_pos, '/', pos, 
    print '(%.2f%%)' % (100.0*errors_pos/pos)
    print 'Errors for negative class', errors_neg, '/', neg,
    print '(%.2f%%)' % (100.0*errors_neg/neg)
    print 'Total errors', (errors_pos + errors_neg), '/', (pos + neg),
    print '(%.2f%%)' % (100.0*(errors_pos + errors_neg)/(pos + neg))

def main():
    '''Simple main program.
    '''
    train, test, sigma, C = get_input()
    train = sp.load(train)['train']
    test = sp.load(test)['test']
    svm = read_solution(train, sigma, C)
    test_svm(svm, test)
    
if __name__ == '__main__':
    main()
