'''
Run the tests based on SVM training.
'''

from subprocess import call, check_output
from platform import architecture
import os
import os.path as path

def print_add(text, log):
    '''Print text and return log with text appended.
    '''
    print text
    return log + text + '\n'

# Tests with sizes and C values
DimC = {
    'adult_01065.svm': (1065,  '1.0'),
    'adult_02265.svm': (2265,  '1.0'),
    'adult_03185.svm': (3185,  '1.0'),
    'adult_06370.svm': (6370,  '1.0'),
    'adult_12740.svm': (12740, '1.0'),
    'mnist_00800.svm': (800,   '10.0'),
    'mnist_01600.svm': (1600,  '10.0'),
    'mnist_03200.svm': (3200,  '10.0'),
    'mnist_06400.svm': (6400,  '10.0'),
    'mnist_11702.svm': (11702, '10.0')
    }
tests = DimC.keys()
tests.sort()

# Projections
projections = ['newton', 'newton-nc', 'secant', 'blg', 'fix', 'search',
               'newtonwarm', 'newtonwarm-nc', 'secantwarm', 'blgwarm']

# Working directories
results_dir = 'test_results'
data_dir = 'data_scripts'
opt_dir =  '../../../lib'
cur_dir = path.realpath('.')

# Verify if all test files are available.
for t in tests:
    if t not in os.listdir(data_dir):
        os.chdir(data_dir)
        call(['python', 'prepare_svm_data.py'])
        os.chdir('..')
        break

# Compile svmma.
call(['make', 'clean'])

# Run the tests.
os.chdir(data_dir)
reps = 1000000
log = ''
for t in tests:
    log = print_add(20*'*' + ' Running test ' + t, log)
    for p in projections:
        optimize_command = ['python', 'optimize.py', '--set_d=1.0',
                            '--set_b=1.0']
        if p[-3:] == '-nc':
            optimize_command.append('--no_compression')
            p = p[:-3]

        os.chdir(opt_dir)
        call(optimize_command)
        os.chdir(cur_dir)
        call(['make'])
        os.chdir(data_dir)
        log = print_add(20*'-' + ' Projection ' + p, log)

        log = print_add(check_output([path.join('..', 'svmma'), t, p,
                                      '%d' % (reps/DimC[t][0]),
                                      DimC[t][1]]), log)
# Save results.
os.chdir(path.join('..', results_dir))
file('test_log', 'w').write(log)

# Generate report in LaTeX Format.
latex = check_output(['python', path.join('..', 'summarize.py'),
                      'test_log'])
out = file('svm_results.tex', 'w')
out.write(latex)
out.close()
os.chdir('..')


